use syntax::ast::{Expr, TokenTree};
use syntax::ext::base::{MacResult, MacEager, DummyResult};
use syntax::parse::token::{InternedString, get_ident};
use syntax::parse::parser::PathParsingMode;

use util::*;


pub fn expand_path<'cx>(cx: &'cx mut ExtCtxt, span: Span, tts: &[TokenTree]) -> Box<MacResult+'cx>  {
	if tts.len() == 0 {
		cx.span_err(span, "ext_path!() requires a argument.");
		return DummyResult::any(span);
	}


	let mut parser = cx.new_parser_from_tts(tts);
	let path = match parser.parse_path(PathParsingMode::LifetimeAndTypesWithoutColons) {
		Ok(path) => path,
		Err(e) => {
			cx.span_err(span, &*format!("Failed to parse path: {}", e));
			return DummyResult::any(span)
		}
	};


	MacEager::expr(experify_path(cx, P(path)))
}

pub fn expand_ty<'cx>(cx: &'cx mut ExtCtxt, span: Span, tts: &[TokenTree]) -> Box<MacResult+'cx>  {
	if tts.len() == 0 {
		cx.span_err(span, "ext_ty!() requires a argument.");
		return DummyResult::any(span);
	}


	let mut parser = cx.new_parser_from_tts(tts);
	let ty = parser.parse_ty();
	MacEager::expr(experify_ty(cx, ty))
}




pub fn experify_ty(cx: &mut ExtCtxt, ty: P<ast::Ty>) -> P<Expr> {
	ty.and_then(|ty| {
		match ty.node {
			ast::TyPath(None, path) => {
				let path = experify_path(cx, P(path));
				return quote_expr!(cx, ::syntax::ext::deriving::generic::ty::Ty::Literal($path));
			},
			_ => unimplemented!(),
		}
	})
}

pub fn experify_lifetime(cx: &mut ExtCtxt, lifetimes: Vec<ast::Lifetime>) -> P<Expr> {
	if lifetimes.is_empty() { return quote_expr!(cx, None); }


	let mut lifetime = String::new();
	let len = lifetimes.len();
	for (i, l) in lifetimes.into_iter().enumerate() {
		lifetime.push_str(l.name.as_str());

		if i != len - 1 { lifetime.push_str(","); }
	}


	quote_expr!(cx, Some($lifetime))
}

pub fn experify_path(cx: &mut ExtCtxt, path: P<ast::Path>) -> P<Expr> {
	let mut paths = vec![];
	let mut params = vec![];
	let mut lifetimes = vec![];
	for seg in &path.segments {

		paths.push(cx.expr_str(path.span, get_ident(seg.identifier)));
	}




	let last = path.segments.iter().last().unwrap();
	match &last.parameters {
		&ast::AngleBracketedParameters(ref data) => {
			for lt in data.lifetimes.iter() {
				lifetimes.push(lt.clone());
			}

			for t in data.types.iter() {
				let ty_expr = experify_ty(cx, t.clone());

				params.push(quote_expr!(cx, ::std::boxed::Box::new($ty_expr)));
			}

			for binding in data.bindings.iter() {
				println!("Binding: {:?}", binding);
			}
		},
		&ast::ParenthesizedParameters(ref data) => {
			println!("Data: {:?}", data);
			cx.span_err(path.span, "Currently unimlemeted!");
			return cx.expr_fail(path.span, InternedString::new(""));
		}
	}


	let paths = vector_expr(cx, path.span, paths);
	let lifetime = experify_lifetime(cx, lifetimes);
	let params = vector_expr(cx, path.span, params);
	let global = cx.expr_bool(path.span, path.global);
	quote_expr!(cx, ::syntax::ext::deriving::generic::ty::Path::new_($paths, $lifetime, $params, $global))
}
